@@ -1,5 +1,5 @@
 #
-#   xts: eXtensible time-series 
+#   xts: eXtensible time-series
 #
 #   Copyright (C) 2008  Jeffrey A. Ryan jeff.a.ryan @ gmail.com
 #
@@ -7,7 +7,7 @@
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
-#   the Free Software Foundation, either version 3 of the License, or
+#   the Free Software Foundation, either version 2 of the License, or
 #   (at your option) any later version.
 #
 #   This program is distributed in the hope that it will be useful,
@@ -35,9 +35,28 @@ periodicity <- function(x, ...) {
   if( timeBased(x) || !is.xts(x) )
     x <- try.xts(x, error='\'x\' needs to be timeBased or xtsible')
 
-  p <- median(diff( .index(x) ))
+  n <- length(.index(x))
+  if( n < 2 ) {
+    res <- list(difftime = structure(0, units='secs', class='difftime'),
+                frequency = 0,
+                start = NA,
+                end = NA,
+                units = 'secs',
+                scale = 'seconds',
+                label = 'second')
+    res <- structure(res, class='periodicity')
 
-  if( is.na(p) ) stop("can not calculate periodicity of 1 observation")
+    if( n == 0 ) {
+      warning("can not calculate periodicity of empty object")
+    } else {
+      warning("can not calculate periodicity of 1 observation")
+      res$start <- start(x)
+      res$end <- end(x)
+    }
+    return(res)
+  } else {
+    p <- median(diff( .index(x) ))
+  }
 
   units <- 'days' # the default if p > hourly
   scale <- 'yearly'# the default for p > quarterly
@@ -152,78 +171,3 @@ function (x, ...)
             "to", x$end, "\n", sep = " "))
     }
 }
-
-`period.apply` <-
-function(x, INDEX, FUN, ...)
-{
-    x <- try.xts(x, error = FALSE)
-    FUN <- match.fun(FUN)
-    xx <- sapply(1:(length(INDEX) - 1), function(y) {
-                   FUN(x[(INDEX[y] + 1):INDEX[y + 1]], ...)
-                })  
-    if(is.vector(xx))
-      xx <- t(xx)
-    xx <- t(xx)
-    if(is.null(colnames(xx)) && NCOL(x)==NCOL(xx))
-      colnames(xx) <- colnames(x)
-    reclass(xx, x[INDEX])  
-}
-
-
-`period.apply.original` <-
-function (x, INDEX, FUN, ...) 
-{
-  x <- use.xts(x,error=FALSE)
-
-  if(!is.xts(x)) {
-    FUN <- match.fun(FUN)
-    xx <- sapply(1:(length(INDEX) - 1), function(y) {
-          FUN(x[(INDEX[y] + 1):INDEX[y + 1]], ...)
-    })
-  } else {
-    FUN <- match.fun(FUN)
-    new.index <- index(x)[INDEX]
-    xx <- sapply(1:(length(INDEX) - 1), function(y) {
-          FUN(x[(INDEX[y] + 1):INDEX[y + 1]], ...)
-    })
-    xx <- xts(xx,new.index)
-    CLASS(xx) <- CLASS(x)
-    xtsAttributes(xx) <- xtsAttributes(x)
-    xx <- reclass(xx)
-  }
-  xx
-}
-
-`apply.daily` <-
-function(x,FUN, ...)
-{
-  ep <- endpoints(x,'days')
-  period.apply(x,ep,FUN, ...)
-}
-`apply.weekly` <-
-function(x,FUN, ...)
-{
-  ep <- endpoints(x,'weeks')
-  period.apply(x,ep,FUN, ...)
-}
-
-`apply.monthly` <-
-function(x,FUN, ...)
-{
-  ep <- endpoints(x,'months')
-  period.apply(x,ep,FUN, ...)
-}
-
-`apply.quarterly` <-
-function(x,FUN, ...)
-{
-  ep <- endpoints(x,'quarters')
-  period.apply(x,ep,FUN, ...)
-}
-
-`apply.yearly` <-
-function(x,FUN, ...)
-{
-  ep <- endpoints(x,'years')
-  period.apply(x,ep,FUN, ...)
-}
